<?php
declare(strict_types=1);

namespace Inti\Wp\Middleware;

use Doctrine\DBAL\Connection;
use Inti\Wp\Plugin\PluginLoader;
use Inti\Wp\Shortcode\ShortcodeLoader;
use Laminas\Diactoros\Response\RedirectResponse;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use function add_action;
use function define;
use function dirname;
use function intval;
use function rtrim;
use function wp_download_language_pack;

/**
 * Class WordPressMiddleware
 *
 * @package Inti\Wp
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class WpBootstrap implements MiddlewareInterface
{
    /** @var bool */
    private static $bootstrapped = false;
    /** @var PluginLoader */
    private $pluginLoader;
    /** @var ShortcodeLoader */
    private $shortcodeLoader;
    /** @var string[] */
    private $globals;

    /**
     * Konstruktor.
     *
     * @param PluginLoader $pluginLoader
     * @param ShortcodeLoader $shortcodeLoader
     * @param array $globals
     */
    public function __construct(PluginLoader $pluginLoader, ShortcodeLoader $shortcodeLoader, array $globals)
    {
        $this->pluginLoader = $pluginLoader;
        $this->shortcodeLoader = $shortcodeLoader;
        $this->globals = $globals;
    }

    /**
     * Process an incoming server request.
     *
     * Processes an incoming server request in order to produce a response.
     * If unable to produce the response itself, it may delegate to the provided
     * request handler to do so.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        if (self::$bootstrapped === false) {
            $this->bootstrap($request);
            self::$bootstrapped = true;
        }

        return $handler->handle($request);
    }

    /**
     * @param ServerRequestInterface $request
     * @noinspection DuplicatedCode
     */
    public function bootstrap(ServerRequestInterface $request): void
    {
        /** @var Connection $db */
        $db = $request->getAttribute('db');
        /** @noinspection PhpInternalEntityUsedInspection */
        $config = $db->getParams();

        define('DB_NAME', $config['dbname']);
        define('DB_USER', $config['user']);
        define('DB_PASSWORD', $config['password']);
        define('DB_HOST', $config['host']);
        define('DB_CHARSET', 'utf8mb4');
        define('DB_COLLATE', 'utf8mb4_croatian_ci');

        define('AUTH_KEY',         'put your unique phrase here');
        define('SECURE_AUTH_KEY',  'put your unique phrase here');
        define('LOGGED_IN_KEY',    'put your unique phrase here');
        define('NONCE_KEY',        'put your unique phrase here');
        define('AUTH_SALT',        'put your unique phrase here');
        define('SECURE_AUTH_SALT', 'put your unique phrase here');
        define('LOGGED_IN_SALT',   'put your unique phrase here');
        define('NONCE_SALT',       'put your unique phrase here');

        /** @noinspection PhpUnusedLocalVariableInspection */
        $table_prefix = 'wp_';

        $config = $request->getAttribute('config');

        define('WP_DEBUG', $config['debug'] ?? false);

        $uri = $request->getUri();
        define('WP_HOME', rtrim((string) $uri->withQuery('')->withPath('/'), '/'));
        define('WP_SITEURL', WP_HOME . '/wp');
        define('WP_CONTENT_DIR', dirname(ABSPATH));
        define('WP_CONTENT_URL', WP_HOME);

        require_once ABSPATH . 'wp-includes/plugin.php';

        add_action('muplugins_loaded', [$this->pluginLoader, 'attachEarly']);
        add_action('plugins_loaded', [$this->pluginLoader, 'attach']);
        add_action('after_setup_theme', [$this->shortcodeLoader, 'register']);

        foreach ($this->globals as $global) {
            global $$global;
        }

        require_once ABSPATH . 'wp-settings.php';
    }

    /**
     * @param ServerRequestInterface $request
     * @return ResponseInterface|null
     * @noinspection PhpUnusedPrivateMethodInspection
     */
    private function wp_install(ServerRequestInterface $request): ?ResponseInterface
    {
        if (defined('WP_INSTALLING')) {
            switch (intval($_GET['step'] ?? '')) {
                case 0:
                    require_once ABSPATH . 'wp-admin/includes/file.php';
                    require_once ABSPATH . 'wp-admin/includes/template.php';
                    require_once ABSPATH . 'wp-admin/includes/translation-install.php';

                    wp_download_language_pack('hr');

                    $uri = $request->getUri();

                    return new RedirectResponse(
                        $uri->withPath($request->getAttribute('BaseUrl') . $uri->getPath())
                            ->withQuery('language=hr&step=1')
                    );
            }
        }

        return null;
    }
}
